# Client
get_database_doc: str
get_default_database_doc: str
start_session_doc: str

# Database
cmd_doc: str
list_collection_names_doc: str

# Collections
bulk_write_doc: str
create_index_doc: str
create_indexes_doc: str
delete_many_doc: str
delete_one_doc: str
drop_doc: str
find_one_doc: str
find_one_and_delete_doc: str
find_one_and_replace_doc: str
find_one_and_update_doc: str
index_information_doc: str
insert_many_doc: str
insert_one_doc: str
mr_doc: str
replace_one_doc: str
update_many_doc: str
update_one_doc: str

# Cursor
cursor_sort_doc: str
where_doc: str

# Client Encryption
create_data_key_doc: str
close_doc: str

# GridFS
gridfs_delete_doc: str
gridfs_download_to_stream_doc: str
gridfs_download_to_stream_by_name_doc: str
gridfs_open_download_stream_doc: str
gridfs_open_download_stream_by_name_doc: str
gridfs_open_upload_stream_doc: str
gridfs_open_upload_stream_with_id_doc: str
gridfs_rename_doc: str
gridfs_upload_from_stream_doc: str
gridfs_upload_from_stream_with_id_doc: str
